#!/bin/bash
#
# This script provides the command and control utility for the
# GigaSpaces Technologies Inc. Service Grid

# The call to setenv.sh can be commented out if necessary.
. `dirname $0`/setenv.sh

scriptDir="$(dirname $0)"
command_line=$*
start=

if [[ $1 == services* ]]; then
    start=1
    case $1 in
    services=LH) XAP_COMPONENT_OPTIONS="$XAP_COMPONENT_OPTIONS $XAP_LUS_OPTIONS"; ;;
    services=GSA) XAP_COMPONENT_OPTIONS="$XAP_COMPONENT_OPTIONS $XAP_GSA_OPTIONS"; ;;
    services=GSM) XAP_COMPONENT_OPTIONS="$XAP_COMPONENT_OPTIONS $XAP_GSM_OPTIONS"; ;;
    services=GSM,LH) XAP_COMPONENT_OPTIONS="$XAP_COMPONENT_OPTIONS $XAP_GSM_OPTIONS"; ;;
    services=GSC) XAP_COMPONENT_OPTIONS="$XAP_COMPONENT_OPTIONS $XAP_GSC_OPTIONS"; ;;
    services=ESM) XAP_COMPONENT_OPTIONS="$XAP_COMPONENT_OPTIONS $XAP_ESM_OPTIONS"; ;;
    services=TM) XAP_COMPONENT_OPTIONS="$XAP_COMPONENT_OPTIONS $XAP_JTX_OPTIONS"; ;;
    esac
    command_line=$*
fi

GS_LIB="$XAP_HOME/lib"

# Function to find a file
getPathForFile() {
    filename="$1"
    if [ -f "$GS_LIB/required/$filename" ] ; then
	located="$GS_LIB/required/$filename"
    else
    echo "Cannot locate $filename in the expected directory structure, exiting"
    exit 1
    fi
}

# Locate the boot strapping jars
getPathForFile xap-common.jar
gsboot=$located

# If the command is to start the Service Grid, invoke the SystemBoot facility.
# Otherwise invoke the CLI to interafce with the product
if [ "$start" = "1" ]; then
    NATIVE_DIR="$GS_LIB/platform/native"
    # Check for running on OS/X
    opSys=`uname -s`
    if [ $opSys = "Darwin" ] ; then
        export DYLD_LIBRARY_PATH=$EXT_LD_LIBRARY_PATH:$LD_LIBRARY_PATH:$NATIVE_DIR
    else
        export LD_LIBRARY_PATH=$EXT_LD_LIBRARY_PATH:$LD_LIBRARY_PATH:$NATIVE_DIR
    fi
# CPP Environment setup
    if [ -f "${XAP_HOME}/cpp/setenv.sh" ]; then
        . ${XAP_HOME}/cpp/setenv.sh
        libpath="-Djava.library.path=$CPP_SPACE_LIB_PATH"
    fi

    script_classpath="-cp $PRE_CLASSPATH:$SIGAR_JARS:$XAP_HOME:$gsboot:$GS_LIB/platform/service-grid/xap-service-grid-boot.jar:$POST_CLASSPATH"
    launchTarget=com.gigaspaces.start.SystemBoot
    "$JAVACMD" ${JAVA_OPTIONS} ${XAP_COMPONENT_OPTIONS} -DagentId=${AGENT_ID} -DgsaServiceID=${GSA_SERVICE_ID} -DenableDynamicLocators=${ENABLE_DYNAMIC_LOCATORS} $script_classpath $libpath ${XAP_OPTIONS} $NETWORK $DEBUG $launchTarget $command_line
else
    XAP_COMPONENT_OPTIONS=$XAP_CLI_OPTIONS
    cliExt="config/services/services.config"
    launchTarget=com.gigaspaces.admin.cli.GS
    script_classpath="-cp $PRE_CLASSPATH:$GS_LIB/platform/cli/*:$GS_JARS:$SPRING_JARS:$MEMORYXTEND_JARS:$SERVICE_GRID_JARS:$POST_CLASSPATH"
    "$JAVACMD" ${JAVA_OPTIONS} ${XAP_COMPONENT_OPTIONS} $script_classpath ${XAP_OPTIONS} $launchTarget $cliExt $command_line

fi